package strategy;

import game.Game;
import game.Strategy;
import game.Utility;
import java.util.Random;

public class BMJoe implements Strategy{
	
	private int numActions;
	private int nextMove;
	private int expectedUtility;
	private Utility gameUtility;
	private int playerAUtility;
	private int playerBUtility;
	Random seed;
	private int opponentToAnnoy;
	
	public BMJoe(Game g, long seed, String options) {
		numActions=g.getNumActions();
		this.seed=new Random(seed);
		nextMove= this.seed.nextInt(numActions);
		gameUtility=g.getUtility();
		int[] utilityArray = {0,0,0};
		utilityArray=gameUtility.getUtility(utilityArray);
		expectedUtility=utilityArray[0];
		playerAUtility=0;
		playerBUtility=0;
		opponentToAnnoy=-1;
}

	public int getAction() {
	    return nextMove;
	  }
	
	public void observeOutcome(int[] actions) {
		int[] utilityArray=gameUtility.getUtility(actions);
		playerAUtility+=utilityArray[1];
		playerBUtility+=utilityArray[2];
		if(utilityArray[0]<expectedUtility)
		{
			//search for best action
			int bestAction=0;
			actions[0]=0;
			int bestUtility=gameUtility.getUtility(actions)[0];
			for(int i=1;i<numActions;i++)
			{
				actions[0]=i;
				utilityArray=gameUtility.getUtility(actions);
				if(utilityArray[0]>bestUtility)
				{
					bestUtility=utilityArray[0];
					bestAction=i;
				}
				if(utilityArray[0]==bestUtility)
				{
					if(seed.nextBoolean())
					{
						bestUtility=utilityArray[0];
						bestAction=i;
					}
				}
			}
			//if found a good action use it
			if(bestUtility>=expectedUtility)
			{
				nextMove=bestAction;
				opponentToAnnoy=-1;
			}
			//if best action is lower than expected utility
			//stick to the player with the highest score in a place where I get the most score
			else
			{
				//check who is the best player
				int actionOfBestOpponent;
				if(opponentToAnnoy==-1)
				{
					if(playerAUtility>playerBUtility)
					{
						actionOfBestOpponent=actions[1];
						opponentToAnnoy=1;
					}
					else if(playerAUtility<playerBUtility)
					{
						actionOfBestOpponent=actions[2];
						opponentToAnnoy=2;
					}
					else //else if both players has the same score choose randomly
					{
						opponentToAnnoy=seed.nextInt(2);
						actionOfBestOpponent=actions[opponentToAnnoy];
					}
				}
				else
				{
					actionOfBestOpponent=actions[opponentToAnnoy];
				}
				//decide where is the best place near the opponent default on the opponent
				actions[0]=actionOfBestOpponent;
				nextMove=actionOfBestOpponent;
				bestUtility=gameUtility.getUtility(actions)[0];
				//one more than the opponent
				actions[0]=action(actionOfBestOpponent+1);
				if(bestUtility<gameUtility.getUtility(actions)[0])
				{
					bestUtility=gameUtility.getUtility(actions)[0];
					nextMove=actions[0];
				}
				else if(bestUtility==gameUtility.getUtility(actions)[0])
				{
					if(seed.nextBoolean())
					{
						bestUtility=gameUtility.getUtility(actions)[0];
						nextMove=actions[0];
					}
				}
				//one less than the opponent
				actions[0]=action(actionOfBestOpponent-1);
				bestUtility=gameUtility.getUtility(actions)[0];
				actions[0]=action(actionOfBestOpponent+1);
				if(bestUtility<gameUtility.getUtility(actions)[0])
				{
					bestUtility=gameUtility.getUtility(actions)[0];
					nextMove=actions[0];
				}
				else if(bestUtility==gameUtility.getUtility(actions)[0])
				{
					if(seed.nextBoolean())
					{
						bestUtility=gameUtility.getUtility(actions)[0];
						nextMove=actions[0];
					}
				}
				
			}
			
		}
		else
		{
			opponentToAnnoy=-1;
		}
	  }
	  
	  
	  
	//returns the action that belongs to a certain number
	public int action(int num)
	{
		while(num<0)
			num+=numActions;
		return num%numActions;
	}

}
