package ca.ualberta.cs.bowling;

import game.Game;
import game.Strategy;

import java.util.Random;
import java.util.Vector;

import lemonade.GeneralizedLemonadeUtility;

/**
 * An 8-hour project.  Code modified from Marty's fancyconstant strategy.
 * @author bowling
 *
 */
public class BowlingStrategy implements Strategy{
    /**
     * The action the player plays
     */
    int constantAction;
    int[] bestActions;

    int[] quality;
    int[] response;

    int scoreAction, scoreResponse;
    
    GeneralizedLemonadeUtility U;
    int totalu;

    protected int collusionFairness(int a, int b) {
	int[] u = U.getUtility(new int[]{a, b});
	return u[0] - u[1];
    }

    protected int collusionSuccess(int a, int b) {
	int min = totalu;
	for (int c=0; c<U.getIslandSize(); c++) {
	    int[] u = U.getUtility(new int[]{a, b, c});
	    if (u[0] + u[1] < min) min = u[0] + u[1];
	}

	return min;
    }

    protected int getQuality(int action){
	int bestv = 0;

	for (int b=0; b < U.getIslandSize(); b++) {
	    int s = collusionSuccess(action, b);
	    int f = collusionFairness(action, b);
	    int v;

	    // If the position is too unfair, then just take it (flip the preference for fairness).
	    // Basically, if one side is worth twice the other, then fighting over that side is better than
	    // accepting the weaker position.
	    if (3 * Math.abs(f) > totalu) f *= (-1);

	    // The objective: prefer high collusion success, fair allocation, and the worst position of such pairs.
	    v = 1000000 * s - 1000 * Math.abs(f) - f;

	    if (v > bestv) { 
		bestv = v;
		response[action] = b;
	    }
	}

	return bestv;
    }


  private void initBestActions(){
    quality = new int[U.getIslandSize()];
    response = new int[U.getIslandSize()];

    totalu = 0;
    for(int i=0; i<U.getIslandSize(); i++) totalu += U.getNumConsumers(i);

    for(int i=0;i<quality.length;i++){
      quality[i]=getQuality(i);
    }
    int maxDensity = quality[0];
    int maxDensityCount = 1;
    for(int i=1;i<quality.length;i++){
      
      if (maxDensity<quality[i]){
        maxDensity=quality[i];
        maxDensityCount=1;
      } else if (maxDensity==quality[i]){
        maxDensityCount++;
      }
    }
    bestActions = new int[maxDensityCount];
    int bestActionIndex = 0;
    for(int i=0;i<quality.length;i++){
      if (quality[i]==maxDensity){
        bestActions[bestActionIndex]=i;
        bestActionIndex++;
      }
    }
  }

  public BowlingStrategy(Game g, long seed, String options) {
    Random r = new Random(seed);
    this.U = (GeneralizedLemonadeUtility)g.getUtility();
    
    initBestActions();
    constantAction = bestActions[r.nextInt(bestActions.length)];
    scoreAction = r.nextInt(48 * 2);
    scoreResponse = 0;
  }
  
  public int getAction() {
      if (scoreAction <= scoreResponse) return response[constantAction];
      else return constantAction;
  }

  public void observeOutcome(int[] actions) {
      scoreAction += this.U.getUtility(new int[] {constantAction, actions[1], actions[2]})[0];
      scoreResponse += this.U.getUtility(new int[] { response[constantAction], actions[1], actions[2]})[0];
  }  
}
